@@ -14,10 +14,12 @@
 #'  direction for sliding. The matrix may be generated using the function \code{\link{define.sliders}}). Likewise, 
 #'  to include semilandmarks 
 #'  on surfaces, one must specify a vector listing which landmarks are to be treated as surface semilandmarks 
-#'  using the "surfaces=" option. The "ProcD=TRUE" option will slide the semilandmarks along their tangent 
-#'  directions using the Procrustes distance criterion, while "ProcD=FALSE" will slide the semilandmarks 
-#'  based on minimizing bending energy. The Procrustes-aligned specimens may be projected into tangent 
-#'  space using the "Proj=TRUE" option. NOTE: Large datasets may exceed the memory limitations of R. 
+#'  using the "surfaces=" option. The "ProcD=FALSE" option (the default) will slide the semilandmarks 
+#'  based on minimizing bending energy, while "ProcD=TRUE" will slide the semilandmarks along their tangent 
+#'  directions using the Procrustes distance criterion. The Procrustes-aligned specimens may be projected into tangent
+#'  space using the "Proj=TRUE" option. 
+#'  The function also outputs a matrix of pairwise Procrustes Distances, which correspond to Euclidean distances between specimens in tangent space if "Proj=TRUE", or to the geodesic distances in shape space if "Proj=FALSE".   
+#'  NOTE: Large datasets may exceed the memory limitations of R. 
 #'
 #'  Generalized Procrustes Analysis (GPA: Gower 1975, Rohlf and Slice 1990) is the primary means by which 
 #'   shape variables are obtained from landmark data (for a general overview of geometric morphometrics see 
@@ -52,12 +54,14 @@
 #' @param Proj A logical value indicating whether or not the Procrustes-aligned specimens should be projected 
 #'   into tangent space 
 #' @param ProcD A logical value indicating whether or not Procrustes distance should be used as the criterion
-#'   for optimizing the positions of semilandmarks
+#'   for optimizing the positions of semilandmarks (if not, bending energy is used)
 #' @param PrinAxes A logical value indicating whether or not to align the shape data by principal axes 
 #' @param max.iter The maximum number of GPA iterations to perform before superimposition is halted.  The final
 #' number of iterations could be larger than this, if curves or surface semilandmarks are involved.
 #' @param curves An optional matrix defining which landmarks should be treated as semilandmarks on boundary 
-#'   curves, and which landmarks specify the tangent directions for their sliding.  
+#'   curves, and which landmarks specify the tangent directions for their sliding.  This matrix is generated automatically
+#'   with \code{\link{readland.shapes}} following digitizing of curves in StereoMorph, or may be generated
+#'   using the function \code{\link{define.sliders}}.
 #' @param surfaces An optional vector defining which landmarks should be treated as semilandmarks on surfaces
 #' @param print.progress A logical value to indicate whether a progress bar should be printed to the screen.  
 #' @keywords analysis
@@ -72,7 +76,11 @@
 #'  \item{iter}{The number of GPA iterations until convergence was found (or GPA halted).}
 #'  \item{points.VCV}{Variance-covariance matrix among Procrustes shape variables.}
 #'  \item{points.var}{Variances of landmark points.}
-#'  \item{consnsus}{The consensus (mean) configuration.}
+#'  \item{consensus}{The consensus (mean) configuration.}
+#'  \item{procD}{Procrustes distance matrix for all specimens (see details). Note that for large data
+#'  sets, R might return a memory allocation error, in which case the error will be suppressed and this component will be NULL.
+#'  For such cases, users can augment memory allocation and create distances with the dist function, independent from gpagen,
+#'  using the coords or data output.}
 #'  \item{p}{Number of landmarks.}
 #'  \item{k}{Number of landmark dimensions.}
 #'  \item{nsliders}{Number of semilandmarks along curves.}
@@ -118,16 +126,18 @@
 #' ###Slider matrix
 #' hummingbirds$curvepts
 #'
-#' # Using Procrustes Distance for sliding
-#' Y.gpa <- gpagen(hummingbirds$land,curves=hummingbirds$curvepts)   
-#' summary(Y.gpa)
-#' plot(Y.gpa)
-#' 
 #' # Using bending energy for sliding
 #' Y.gpa <- gpagen(hummingbirds$land,curves=hummingbirds$curvepts,ProcD=FALSE)   
 #' summary(Y.gpa)
 #' plot(Y.gpa)
 #' 
+#' 
+#' # Using Procrustes Distance for sliding
+#' Y.gpa <- gpagen(hummingbirds$land,curves=hummingbirds$curvepts,ProcD=TRUE)   
+#' summary(Y.gpa)
+#' plot(Y.gpa)
+#' 
+#' 
 #' # Example 3: points, curves and surfaces
 #' data(scallops)
 #' 
@@ -136,8 +146,9 @@
 #' # NOTE can summarize as: summary(Y.gpa)
 #' # NOTE can plot as: plot(Y.gpa) 
 gpagen = function(A, curves=NULL, surfaces=NULL, PrinAxes = TRUE, 
-                  max.iter = NULL, ProcD=TRUE, Proj = TRUE,
+                  max.iter = NULL, ProcD=FALSE, Proj = TRUE,
                   print.progress = TRUE){
+  
   if(inherits(A, "geomorphShapes")) {
     Y <- A$landmarks
     if(any(unlist(lapply(Y, is.na)))) stop("Data matrix contains missing values. Estimate these first (see 'estimate.missing').")
@@ -146,12 +157,29 @@ gpagen = function(A, curves=NULL, surfaces=NULL, PrinAxes = TRUE,
     p <- A$p
     k <- A$k
     
+    spec.names <- names(Y)
+    p.names <- dimnames(Y[[1]])[[1]]
+    k.names <- c("X", "Y", "Z")[1:k] 
+    
   } else {
     
     if(!is.array(A)) stop("Coordinates must be a 3D array")
     if(length(dim(A)) != 3) stop("Coordinates array does not have proper dimensions")
     if(any(is.na(A))) stop("Data matrix contains missing values. Estimate these first (see 'estimate.missing').")
     n <- dim(A)[[3]]; p <- dim(A)[[1]]; k <- dim(A)[[2]]
+    
+    spec.names <- 1:n
+    p.names <- 1:p
+    k.names <- c("X", "Y", "Z")[1:k] 
+    
+    dim.names <- dimnames(A)
+    if(length(dim.names) != 0) {
+      dim.name.check <- sapply(1:length(dim.names), is.null)
+      if(!dim.name.check[[1]]) spec.names <- dim.names[[1]]
+      if(!dim.name.check[[2]]) spec.names <- dim.names[[2]]
+      if(!dim.name.check[[3]]) spec.names <- dim.names[[3]]
+    }
+    
     Y <- lapply(1:n, function(j) A[,,j])
   }
   
@@ -183,30 +211,29 @@ gpagen = function(A, curves=NULL, surfaces=NULL, PrinAxes = TRUE,
   
   coords <- gpa$coords
   M <- gpa$consensus
+  dimnames(M) <- list(p.names, k.names)
+    
   if (Proj == TRUE) {
     coords <- orp(coords)
     M <- Reduce("+",coords)/n
-    colnames(M) <- dimnames(A)[[2]]
-    rownames(M) <- dimnames(A)[[1]]
+    dimnames(M) <- list(p.names, k.names)
   }
   Csize <- gpa$CS
+  names(Csize) <- spec.names
   iter <- gpa$iter
   pt.var <- Reduce("+",Map(function(y) y^2/n, coords))
   coords <- simplify2array(coords)
-  if(inherits(A, "geomorphShapes")) dimnames(coords)[[3]] <- names(A$landmarks) else
-    dimnames(coords) <- dimnames(A)
-  pt.VCV <- var(two.d.array(coords))
-  rownames(pt.var) <- dimnames(coords)[[1]]
-  colnames(pt.var) <- if(k==3) c("Var.X", "Var.Y", "Var.Z") else 
-    c("Var.X", "Var.Y") 
-  pt.names <- if(k==3) paste(c("X","Y","Z"), sort(rep(1:p,k)), sep=".") else
-    paste(c("X","Y"), sort(rep(1:p,k)), sep=".")
-  if(is.null(colnames(pt.VCV))) rownames(pt.VCV) <- 
-    colnames(pt.VCV) <- pt.names
-  if(is.null(colnames(M))) colnames(M) <- if(k==3) c("X", "Y", "Z") else c("X", "Y")
+  dimnames(coords) <- list(p.names, k.names, spec.names)
   two.d.coords = two.d.array(coords)
-  if(is.null(colnames(two.d.coords))) colnames(two.d.coords) <- pt.names
-  names(Csize) <- dimnames(A)[[3]]
+  rownames(two.d.coords) <- spec.names
+  pt.VCV <- var(two.d.coords)
+  rownames(pt.var) <- p.names
+  colnames(pt.var) <- c("Var.X", "Var.Y", "Var.Z")[1:k]
+
+  if(is.null(colnames(M))) colnames(M) <- c("X", "Y", "Z")[1:k] 
+
+  procD <- try(dist(two.d.coords), silent = TRUE)
+  if(inherits(procD, "try-error")) procD <- NULL
   if(!is.null(curves) || !is.null(surf)) {
     nsliders <- nrow(curves)
     nsurf <- length(surf)
@@ -217,11 +244,12 @@ gpagen = function(A, curves=NULL, surfaces=NULL, PrinAxes = TRUE,
     smeth <- NULL
   }
   if(is.null(nsliders)) nsliders <- 0; if(is.null(nsurf)) nsurf <- 0
+
   out <- list(coords=coords, Csize=Csize, 
               iter=iter, 
               points.VCV = pt.VCV, points.var = pt.var, 
-              consensus = M, p=p,k=k, 
-              nsliders=nsliders, nsurf = nsurf,
+              consensus = M, procD = procD, 
+              p=p,k=k, nsliders=nsliders, nsurf = nsurf,
               data = data.frame(coords = two.d.coords, Csize = Csize),
               Q = gpa$Q, slide.method = smeth, call= match.call())
   class(out) <- "gpagen"
